import { s as Report, c as ConfigLoader, H as HtmlValidate, a as ConfigData } from './core.js';
export { A as AttributeData, a2 as AttributeEvent, f as CSSStyleDeclaration, af as CompatibilityOptions, a4 as ConditionalEvent, C as Config, a as ConfigData, b as ConfigError, c as ConfigLoader, Q as ConfigReadyEvent, a7 as DOMLoadEvent, a8 as DOMReadyEvent, x as DeferredMessage, a5 as DirectiveEvent, a6 as DoctypeEvent, D as DynamicValue, a1 as ElementReadyEvent, O as Event, K as EventCallback, E as EventDump, J as EventHandler, ab as FileSystemConfigLoader, ac as Formatter, e as HtmlElement, H as HtmlValidate, aa as ListenEventMap, L as Location, u as Message, m as MetaCopyableProperty, M as MetaData, j as MetaDataTable, k as MetaElement, l as MetaTable, i as NestedError, N as NodeClosed, G as Parser, F as Plugin, P as ProcessElementContext, s as Report, t as Reporter, w as Result, R as Rule, n as RuleDocumentation, h as SchemaValidationError, S as Severity, r as Source, V as SourceReadyEvent, d as StaticConfigLoader, $ as TagCloseEvent, _ as TagEndEvent, Z as TagOpenEvent, a0 as TagReadyEvent, Y as TagStartEvent, B as TemplateExtractor, q as TextClassification, T as TextNode, g as TokenDump, X as TokenEvent, y as TransformContext, z as Transformer, a9 as TriggerEventMap, U as UserError, a3 as WhitespaceEvent, W as WrappedError, o as classifyNodeText, ae as compatibilityCheck, p as configPresets, ad as formatterFactory, I as ruleExists, v as version } from './core.js';
import 'ajv';

interface ExpandOptions {
    /**
     * Working directory. Defaults to `process.cwd()`.
     */
    cwd?: string;
    /**
     * List of extensions to search for when expanding directories. Extensions
     * should be passed without leading dot, e.g. "html" instead of ".html".
     */
    extensions?: string[];
}

interface InitResult {
    filename: string;
}

interface CLIOptions {
    configFile?: string;
    rules?: string | string[];
}
/**
 * @public
 */
declare class CLI {
    private options;
    private config;
    private loader;
    private ignored;
    /**
     * Create new CLI helper.
     *
     * Can be used to create tooling with similar properties to bundled CLI
     * script.
     */
    constructor(options?: CLIOptions);
    /**
     * Returns list of files matching patterns and are not ignored. Filenames will
     * have absolute paths.
     *
     * @public
     */
    expandFiles(patterns: string[], options?: ExpandOptions): string[];
    getFormatter(formatters: string): (report: Report) => string;
    /**
     * Initialize project with a new configuration.
     *
     * A new `.htmlvalidate.json` file will be placed in the path provided by
     * `cwd`.
     */
    init(cwd: string): Promise<InitResult>;
    /**
     * Searches ".htmlvalidateignore" files from filesystem and returns `true` if
     * one of them contains a pattern matching given filename.
     */
    isIgnored(filename: string): boolean;
    /**
     * Clear cache.
     *
     * Previously fetched [[HtmlValidate]] instances must either be fetched again
     * or call [[HtmlValidate.flushConfigCache]].
     */
    clearCache(): void;
    /**
     * Get HtmlValidate instance with configuration based on options passed to the
     * constructor.
     *
     * @internal
     */
    getLoader(): ConfigLoader;
    /**
     * Get HtmlValidate instance with configuration based on options passed to the
     * constructor.
     *
     * @public
     */
    getValidator(): HtmlValidate;
    /**
     * @internal
     */
    getConfig(): ConfigData;
}

export { CLI };
